@@ -351,10 +351,12 @@ static void gnome_wp_remove_wp (gchar * key, GnomeWPItem * item,
 
   if (item->rowref != NULL && item->deleted == FALSE) {
     path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_model_get_iter (capplet->model, &iter, path);
-    gtk_tree_path_free (path);
+    if (path != NULL) {
+      gtk_tree_model_get_iter (capplet->model, &iter, path);
+      gtk_tree_path_free (path);
 
-    gtk_list_store_remove (GTK_LIST_STORE (capplet->model), &iter);
+      gtk_list_store_remove (GTK_LIST_STORE (capplet->model), &iter);
+    }
   }
 }
 
@@ -482,7 +484,7 @@ static void gnome_wp_shade_type_changed (GtkMenuShell * shell,
 }
 
 static void gnome_wp_color_changed (GnomeWPCapplet * capplet,
-				    gboolean primary) {
+				    gboolean update) {
   GnomeWPItem * item = NULL;
   GtkTreeIter iter;
   GtkTreeModel * model;
@@ -503,26 +505,29 @@ static void gnome_wp_color_changed (GnomeWPCapplet * capplet,
     return;
   }
 
-  if (primary) {
-    gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (capplet->pc_picker),
-				&item->pcolor->red,
-				&item->pcolor->green,
-				&item->pcolor->blue, NULL);
-    item->pri_color = g_strdup_printf ("#%02X%02X%02X",
-				       item->pcolor->red >> 8,
-				       item->pcolor->green >> 8,
-				       item->pcolor->blue >> 8);
+  g_free (item->pri_color);
+  gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (capplet->pc_picker),
+			      &item->pcolor->red,
+			      &item->pcolor->green,
+			      &item->pcolor->blue, NULL);
+  item->pri_color = g_strdup_printf ("#%02X%02X%02X",
+				     item->pcolor->red >> 8,
+				     item->pcolor->green >> 8,
+				     item->pcolor->blue >> 8);
+
+  g_free (item->sec_color);
+  gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (capplet->sc_picker),
+			      &item->scolor->red,
+			      &item->scolor->green,
+			      &item->scolor->blue, NULL);
+  item->sec_color = g_strdup_printf ("#%02X%02X%02X",
+				     item->scolor->red >> 8,
+				     item->scolor->green >> 8,
+				     item->scolor->blue >> 8);
+
+  if (update) {
     gconf_client_set_string (capplet->client, WP_PCOLOR_KEY,
 			     item->pri_color, NULL);
-  } else {
-    gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (capplet->sc_picker),
-				&item->scolor->red,
-				&item->scolor->green,
-				&item->scolor->blue, NULL);
-    item->sec_color = g_strdup_printf ("#%02X%02X%02X",
-				       item->scolor->red >> 8,
-				       item->scolor->green >> 8,
-				       item->scolor->blue >> 8);
     gconf_client_set_string (capplet->client, WP_SCOLOR_KEY,
 			     item->sec_color, NULL);
   }
@@ -544,16 +549,10 @@ static void gnome_wp_color_changed (GnomeWPCapplet * capplet,
   g_object_unref (pixbuf);
 }
 
-static void gnome_wp_pcolor_changed (GtkWidget * widget,
-				     guint r, guint g, guint b, guint a,
-				     GnomeWPCapplet * capplet) {
-  gnome_wp_color_changed (capplet, TRUE);
-}
-
 static void gnome_wp_scolor_changed (GtkWidget * widget,
 				     guint r, guint g, guint b, guint a,
 				     GnomeWPCapplet * capplet) {
-  gnome_wp_color_changed (capplet, FALSE);
+  gnome_wp_color_changed (capplet, TRUE);
 }
 
 static void gnome_wp_remove_wallpaper (GtkWidget * widget,
@@ -627,24 +626,8 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 				item->scolor->green,
 				item->scolor->blue, 65535);
   } else if (strcmp (style, "none") != 0) {
-    item = gnome_wp_item_new (imagepath, capplet->wphash, capplet->thumbs);
-    if (item != NULL) {
-      wp_props_load_wallpaper (item->filename, item, capplet);
-
-      gnome_wp_capplet_scroll_to_item (capplet, item);
-
-      gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (capplet->pc_picker),
-				  item->pcolor->red,
-				  item->pcolor->green,
-				  item->pcolor->blue, 65535);
-      gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (capplet->sc_picker),
-				  item->scolor->red,
-				  item->scolor->green,
-				  item->scolor->blue, 65535);
-
-      gnome_wp_option_menu_set (capplet, item->options, FALSE);
-      gnome_wp_option_menu_set (capplet, item->shade_type, TRUE);
-    }
+    item = gnome_wp_add_image (capplet, imagepath);
+    gnome_wp_capplet_scroll_to_item (capplet, item);
   }
 
   item = g_hash_table_lookup (capplet->wphash, "(none)");
@@ -666,6 +649,11 @@ static gboolean gnome_wp_load_stuffs (void * data) {
   g_free (imagepath);
   g_free (style);
 
+  if (capplet->uri_list) {
+    gnome_wp_add_images (capplet, capplet->uri_list);
+    g_slist_free (capplet->uri_list);
+  }
+
   return FALSE;
 }
 
@@ -714,12 +702,8 @@ static void gnome_wp_file_changed (GConfClient * client, guint id,
       if (item != NULL) {
 	gnome_wp_capplet_scroll_to_item (capplet, item);
       } else {
-	item = gnome_wp_item_new (wpfile, capplet->wphash, capplet->thumbs);
-	if (item != NULL) {
-	  wp_props_load_wallpaper (item->filename, item, capplet);
-      
-	  gnome_wp_capplet_scroll_to_item (capplet, item);
-	}
+	item = gnome_wp_add_image (capplet, wpfile);
+	gnome_wp_capplet_scroll_to_item (capplet, item);
       }
     }
     g_free (wpfile);
@@ -788,7 +772,7 @@ static void gnome_wp_color1_changed (GConfClient * client, guint id,
 			      color.green,
 			      color.blue, 65535);
 
-  gnome_wp_color_changed (capplet, TRUE);
+  gnome_wp_color_changed (capplet, FALSE);
 }
 
 static void gnome_wp_color2_changed (GConfClient * client, guint id,
@@ -909,8 +893,36 @@ static void set_accessible_name (GtkWidget *widget, const gchar *name) {
   if (name)
     atk_object_set_name (obj, name);
 } 
+
+static void gnome_wp_update_preview (GtkFileChooser *chooser,
+				     GnomeWPCapplet *capplet) {
+  gchar *uri;
+
+  uri = gtk_file_chooser_get_preview_uri (chooser);
+
+  if (uri) {
+    GdkPixbuf *pixbuf;
+    gchar *mime_type;
+
+    mime_type = gnome_vfs_get_mime_type (uri);
+    pixbuf = gnome_thumbnail_factory_generate_thumbnail (capplet->thumbs,
+							 uri,
+							 mime_type);
+
+    if(pixbuf != NULL) {
+      gtk_image_set_from_pixbuf (GTK_IMAGE (capplet->image), pixbuf);
+      g_object_unref (pixbuf);
+    } else {
+      gtk_image_set_from_stock (GTK_IMAGE (capplet->image),
+				"gtk-dialog-question",
+				GTK_ICON_SIZE_DIALOG);
+    }
+    g_free (mime_type);
+  }
+  gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
+}
   
-static void wallpaper_properties_init (void) {
+static void wallpaper_properties_init (poptContext ctx) {
   GnomeWPCapplet * capplet;
   GladeXML * dialog;
   GtkWidget * menu, * label;
@@ -922,6 +934,7 @@ static void wallpaper_properties_init (void) {
   GdkPixbuf * pixbuf;
   GdkCursor * cursor;
   gchar * icofile;
+  const gchar ** args;
 
   gtk_rc_parse_string ("style \"wp-tree-defaults\" {\n"
 		       "  GtkTreeView::horizontal-separator = 6\n"
@@ -943,9 +956,9 @@ static void wallpaper_properties_init (void) {
 					 WP_DELAY_KEY,
 					 NULL);
   gconf_client_add_dir (capplet->client, WP_KEYBOARD_PATH,
-			GCONF_CLIENT_PRELOAD_NONE, NULL);
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (capplet->client, WP_PATH_KEY,
-			GCONF_CLIENT_PRELOAD_NONE, NULL);
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
   gconf_client_notify_add (capplet->client,
 			   WP_DELAY_KEY,
@@ -1105,6 +1118,8 @@ static void wallpaper_properties_init (void) {
     gtk_box_pack_start (GTK_BOX (mbox), capplet->sitem, FALSE, FALSE, 0);
     gtk_widget_show (capplet->sitem);
   }
+  g_free (icofile);
+
   label = gtk_label_new (_("Scaled"));
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_box_pack_start (GTK_BOX (mbox), label, TRUE, TRUE, 0);
@@ -1126,6 +1141,8 @@ static void wallpaper_properties_init (void) {
     gtk_box_pack_start (GTK_BOX (mbox), capplet->witem, FALSE, FALSE, 0);
     gtk_widget_show (capplet->witem);
   }
+  g_free (icofile);
+
   label = gtk_label_new (_("Tiled"));
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_box_pack_start (GTK_BOX (mbox), label, TRUE, TRUE, 0);
@@ -1206,7 +1223,7 @@ static void wallpaper_properties_init (void) {
 
   capplet->pc_picker = glade_xml_get_widget (dialog,"pcpicker");
   g_signal_connect (G_OBJECT (capplet->pc_picker), "color_set",
-		    G_CALLBACK (gnome_wp_pcolor_changed), capplet);
+		    G_CALLBACK (gnome_wp_scolor_changed), capplet);
 
   capplet->sc_picker = glade_xml_get_widget (dialog,"scpicker");
   g_signal_connect (G_OBJECT (capplet->sc_picker), "color_set",
@@ -1222,6 +1239,17 @@ static void wallpaper_properties_init (void) {
   gdk_window_set_cursor (capplet->window->window, cursor);
   gdk_cursor_unref (cursor);
 
+  args = poptGetArgs (ctx);
+  if (args != NULL) {
+    const gchar ** p;
+
+    for (p = args; *p != NULL; p++) {
+      capplet->uri_list = g_slist_append (capplet->uri_list, (gchar *) *p);
+    }
+  }
+
+  poptFreeContext (ctx);
+
   g_idle_add (gnome_wp_load_stuffs, capplet);
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (capplet->treeview));
@@ -1229,20 +1257,36 @@ static void wallpaper_properties_init (void) {
 		    G_CALLBACK (gnome_wp_props_wp_selected), capplet);
 
   /* Create the file chooser dialog stuff here */
-  capplet->filesel = gtk_file_chooser_dialog_new (_("Add Wallpaper"),
-						  GTK_WINDOW (capplet->window),
-						  GTK_FILE_CHOOSER_ACTION_OPEN,
-						  GTK_STOCK_CANCEL,
-						  GTK_RESPONSE_CANCEL,
-						  GTK_STOCK_OPEN,
-						  GTK_RESPONSE_OK,
-						  NULL);
+  capplet->filesel = gtk_file_chooser_dialog_new_with_backend (_("Add Wallpaper"),
+							       GTK_WINDOW (capplet->window),
+							       GTK_FILE_CHOOSER_ACTION_OPEN,
+							       "gtk+",
+							       GTK_STOCK_CANCEL,
+							       GTK_RESPONSE_CANCEL,
+							       GTK_STOCK_OPEN,
+							       GTK_RESPONSE_OK,
+							       NULL);
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (capplet->filesel),
 					TRUE);
+
+  gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (capplet->filesel),
+					  FALSE);
+
+  capplet->image = gtk_image_new ();
+  gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (capplet->filesel),
+				       capplet->image);
+  gtk_widget_set_size_request (capplet->image, 128, -1);
+  
+  gtk_widget_show (capplet->image);
+
+  g_signal_connect (capplet->filesel, "update-preview",
+		    G_CALLBACK (gnome_wp_update_preview), capplet);
 }
 
 gint main (gint argc, gchar *argv[]) {
-  GnomeProgram * proggie;
+  GnomeProgram * program;
+  GValue context = { 0 };
+  poptContext ctx;
 
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -1250,11 +1294,16 @@ gint main (gint argc, gchar *argv[]) {
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  proggie = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
+  program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
 				argc, argv, GNOME_PARAM_POPT_TABLE,
 				NULL, NULL);
 
-  wallpaper_properties_init ();
+  g_object_get_property (G_OBJECT (program), GNOME_PARAM_POPT_CONTEXT,
+			 g_value_init (&context, G_TYPE_POINTER));
+  ctx = g_value_get_pointer (&context);
+
+  wallpaper_properties_init (ctx);
+
   gtk_main ();
   
   return 0;
