@@ -418,7 +418,7 @@ handle_change_signal (GnomeThemeType       type,
 		      GnomeThemeChangeType change_type,
 		      GnomeThemeElement    element)
 {
-#if DEBUG
+#ifdef DEBUG
   gchar *type_str = NULL;
   gchar *element_str = NULL;
 #endif
@@ -441,7 +441,7 @@ handle_change_signal (GnomeThemeType       type,
     (* callback_data->func) (uri, callback_data->data);
   }
 
-#if DEBUG
+#ifdef DEBUG
   if (change_type == GNOME_THEME_CHANGE_CREATED)
     type_str = "created";
   else if (change_type == GNOME_THEME_CHANGE_CHANGED)
@@ -527,7 +527,7 @@ update_theme_index (GnomeVFSURI       *index_uri,
 	    theme_info->has_metacity = TRUE;
 
 	  g_hash_table_insert (theme_hash_by_uri, g_strdup (common_theme_dir), theme_info);
-	  add_data_to_hash_by_name (theme_hash_by_name, g_strdup (theme_info->name), theme_info);
+	  add_data_to_hash_by_name (theme_hash_by_name, theme_info->name, theme_info);
 	  handle_change_signal (GNOME_THEME_TYPE_REGULAR, theme_info, GNOME_THEME_CHANGE_CREATED, key_element);
 	}
     }
@@ -687,7 +687,7 @@ update_common_theme_dir_index (GnomeVFSURI *theme_index_uri,
       if (theme_exists)
 	{
 	  g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
-	  add_data_to_hash_by_name (hash_by_name, g_strdup (name), theme_info);
+	  add_data_to_hash_by_name (hash_by_name, name, theme_info);
 	  handle_change_signal (icon_theme?GNOME_THEME_TYPE_ICON:GNOME_THEME_TYPE_METATHEME,
 				theme_info, GNOME_THEME_CHANGE_CREATED, 0);
 	}
@@ -705,7 +705,7 @@ update_common_theme_dir_index (GnomeVFSURI *theme_index_uri,
 	  if (cmp != 0)
 	    {
 	      g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
-	      add_data_to_hash_by_name (hash_by_name, g_strdup (name), theme_info);
+	      add_data_to_hash_by_name (hash_by_name, name, theme_info);
 	      handle_change_signal (icon_theme?GNOME_THEME_TYPE_ICON:GNOME_THEME_TYPE_METATHEME,
 				    theme_info, GNOME_THEME_CHANGE_CHANGED, 0);
 	      if (icon_theme)
@@ -953,6 +953,7 @@ top_theme_dir_changed (GnomeVFSMonitorHandle *handle,
 	  g_hash_table_remove (handle_hash, name);
 	  g_free (monitor_data);
 	}
+      g_free (name);
     }
   gnome_vfs_uri_unref (common_theme_dir_uri);
 }
@@ -1004,6 +1005,7 @@ top_icon_theme_dir_changed (GnomeVFSMonitorHandle    *handle,
 	  g_hash_table_remove (handle_hash, name);
 	  g_free (monitor_data);
 	}
+      g_free (name);
     }
   gnome_vfs_uri_unref (common_icon_theme_dir_uri);
 }
@@ -1185,7 +1187,7 @@ real_add_top_theme_dir_monitor (GnomeVFSURI  *uri,
    * use it to remove the monitor handles when a dir is removed.
    */
   tuple = g_new0 (CallbackTuple, 1);
-  tuple->handle_hash = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
+  tuple->handle_hash = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, NULL);
   tuple->priority = priority;
 
   /* Check the URI */
@@ -1380,6 +1382,8 @@ gnome_theme_icon_info_new (void)
 void
 gnome_theme_icon_info_free (GnomeThemeIconInfo *icon_theme_info)
 {
+  g_free (icon_theme_info->name);
+  g_free (icon_theme_info->path);
   g_free (icon_theme_info);
 }
 
